#include <main/k2audio.h>
#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_task_wdt.h"
#include "esp_system.h"
#include "driver/gpio.h"
#include "esp_log.h"

void setGPIOs(void);

#define TAG "K2Tasks :"

#define PAUSE_REQ  8



void blinky(void *pvParameter) {
	static bool isBoot = true;
	static uint8_t LED_count = 20;
	static uint8_t LED_state = 0;
	uint8_t onTime = 2;
	uint8_t offTime = 2;
	uint8_t bootCount = 0;
	uint8_t gpio_counter = 0;
	bool pause_required =0;
		uint8_t pauseCount = 0;

	while (1) {

		if (RunFlags.connected) {
			if ((isBoot) && (bootCount > 30)) {
				if (!RunFlags.discoveryDataGenerated) {
					buildDiscovery(0x00);
				}
				RunFlags.discDataReady = true;
				isBoot = false;
				RunFlags.appValuesChanged = true;
				//mdns_check_button();
			}
			bootCount++;

		}

		switch (LEDStatus) {
		case AP_ON:
			onTime = 2; // very fast even flash
			offTime = 2;
			pause_required = true;
			break;
		case AP_CON:
			onTime = 12;
			offTime = 2; // slow flash mostly off
			pause_required = false;
			break;
		case STA_ON:
			onTime = 4; /// fastish even flash
			offTime = 4;
			pause_required = false;
			break;
		case STA_CON:
			onTime = 2; // slow flash mostly on
			offTime = 12;
			pause_required = false;
			break;
		default:
			break;
		}
	//	ESP_LOGI(TAG,"LED status = %d ontime = %d offTime = %d", LEDStatus, onTime, offTime);

		if (pause_required){
			if(pauseCount > PAUSE_REQ){
				offTime = 24;
				pauseCount = 0;
			}
			pauseCount++;

		}

		// leds
		LED_count++;
		/* Blink off time (output low) */
		if (LED_state == 0) {
			if (LED_count > onTime) {
			//	ESP_LOGI(TAG,"OFF Led_count = %d", LED_count);
				gpio_set_level(STATUS_GPIO, 0);
				LED_count = 0;
				LED_state = 1;
			}

		} else {
			// blink on
			if (LED_count > offTime) {
			//	ESP_LOGI(TAG,"ON Led_count = %d", LED_count);
				gpio_set_level(STATUS_GPIO, 1);
				LED_count = 0;
				LED_state = 0;

			}

		}

		gpio_counter++;

		if  (gpio_counter > 4){
			setGPIOs();
			gpio_counter = 0;
			//ESP_LOGI(TAG," GPIOs set or cleared");

		}


		if ((RunFlags.appValuesChanged) || (RunFlags.resetRequired)) {
			if (RunFlags.resetRequired) {
				if (RunFlags.appValuesChanged) {
					resetSystem(1, 0); // reset but save first;
				} else {
					// just reboot/
					resetSystem(0, 0);
				}
			} else {
				saveAppValues();
			}
		}

		vTaskDelay(250 / portTICK_RATE_MS); // 1/4 second task delay
		esp_task_wdt_reset();
	}

}

void setGPIOs(){


	if (AppValues.STORED_FLAGS.gpio_use_for_display == false) {

		if (AppValues.GPIOS.gpio_00_pulsed == 1) {
			PulsedGpioState.gpio_00 = !PulsedGpioState.gpio_00;
			gpio_set_level(GPIO_NUM_0, PulsedGpioState.gpio_00);
		} else {
			gpio_set_level(GPIO_NUM_0, AppValues.GPIOS.gpio_00_state);
		}

		if (AppValues.GPIOS.gpio_05_pulsed == 1) {
			PulsedGpioState.gpio_05 = !PulsedGpioState.gpio_05;
			gpio_set_level(GPIO_NUM_5, PulsedGpioState.gpio_05);
		} else {
			gpio_set_level(GPIO_NUM_5, AppValues.GPIOS.gpio_05_state);
		}

		if (AppValues.GPIOS.gpio_13_pulsed == 1) {
			PulsedGpioState.gpio_13 = !PulsedGpioState.gpio_13;
			gpio_set_level(GPIO_NUM_13, PulsedGpioState.gpio_13);
		} else {
			gpio_set_level(GPIO_NUM_13, AppValues.GPIOS.gpio_13_state);
		}

		if (AppValues.GPIOS.gpio_14_pulsed == 1) {
			PulsedGpioState.gpio_14 = !PulsedGpioState.gpio_14;
			gpio_set_level(GPIO_NUM_14, PulsedGpioState.gpio_14);
		} else {
			gpio_set_level(GPIO_NUM_14, AppValues.GPIOS.gpio_14_state);
		}

		if (AppValues.GPIOS.gpio_15_pulsed == 1) {
			PulsedGpioState.gpio_15 = !PulsedGpioState.gpio_15;
			gpio_set_level(GPIO_NUM_15, PulsedGpioState.gpio_15);
		} else {
			gpio_set_level(GPIO_NUM_15, AppValues.GPIOS.gpio_15_state);
		}

		if (AppValues.GPIOS.gpio_19_pulsed == 1) {
			PulsedGpioState.gpio_19 = !PulsedGpioState.gpio_19;
			gpio_set_level(GPIO_NUM_19, PulsedGpioState.gpio_19);
		} else {
			gpio_set_level(GPIO_NUM_19, AppValues.GPIOS.gpio_19_state);
		}

	}

	if (AppValues.STORED_FLAGS.gpio_use_for_i2c == false) {

		if (AppValues.GPIOS.gpio_21_pulsed == 1) {
			PulsedGpioState.gpio_21 = !PulsedGpioState.gpio_21;
			gpio_set_level(GPIO_NUM_21, PulsedGpioState.gpio_21);
		} else {
			gpio_set_level(GPIO_NUM_21, AppValues.GPIOS.gpio_21_state);
		}

		if (AppValues.GPIOS.gpio_22_pulsed == 1) {
			PulsedGpioState.gpio_22 = !PulsedGpioState.gpio_22;
			gpio_set_level(GPIO_NUM_22, PulsedGpioState.gpio_22);
		} else {
			gpio_set_level(GPIO_NUM_22, AppValues.GPIOS.gpio_22_state);
		}

	}

	if (AppValues.GPIOS.gpio_04_pulsed == 1) {
		PulsedGpioState.gpio_04 = !PulsedGpioState.gpio_04;
		gpio_set_level(GPIO_NUM_4, PulsedGpioState.gpio_04);
	} else {
		gpio_set_level(GPIO_NUM_4, AppValues.GPIOS.gpio_04_state);
	}

	if (AppValues.GPIOS.gpio_12_pulsed == 1) {
		PulsedGpioState.gpio_12 = !PulsedGpioState.gpio_12;
		gpio_set_level(GPIO_NUM_12, PulsedGpioState.gpio_12);
	} else {
		gpio_set_level(GPIO_NUM_12, AppValues.GPIOS.gpio_12_state);
	}

	if (AppValues.GPIOS.gpio_16_pulsed == 1) {
		PulsedGpioState.gpio_16 = !PulsedGpioState.gpio_16;
		gpio_set_level(GPIO_NUM_16, PulsedGpioState.gpio_16);
	} else {
		gpio_set_level(GPIO_NUM_16, AppValues.GPIOS.gpio_16_state);
	}

	if (AppValues.GPIOS.gpio_17_pulsed == 1) {
		PulsedGpioState.gpio_17 = !PulsedGpioState.gpio_17;
		gpio_set_level(GPIO_NUM_17, PulsedGpioState.gpio_17);
	} else {
		gpio_set_level(GPIO_NUM_17, AppValues.GPIOS.gpio_17_state);
	}

	if (AppValues.GPIOS.gpio_18_pulsed == 1) {

		PulsedGpioState.gpio_18 = !PulsedGpioState.gpio_18;
		gpio_set_level(GPIO_NUM_18, PulsedGpioState.gpio_18);
	} else {
		gpio_set_level(GPIO_NUM_18, AppValues.GPIOS.gpio_18_state);
	}

	if (AppValues.GPIOS.gpio_23_pulsed == 1) {
		PulsedGpioState.gpio_23 = !PulsedGpioState.gpio_23;
		gpio_set_level(GPIO_NUM_23, PulsedGpioState.gpio_23);
	} else {
		ESP_LOGI(TAG, "gpio23 state is %d", AppValues.GPIOS.gpio_23_state);
		gpio_set_level(GPIO_NUM_23, 1);
	}

	if (AppValues.GPIOS.gpio_25_pulsed == 1) {
		PulsedGpioState.gpio_25 = !PulsedGpioState.gpio_25;
		gpio_set_level(GPIO_NUM_25, PulsedGpioState.gpio_25);
	} else {
		gpio_set_level(GPIO_NUM_25, 1);
	}

	if (AppValues.GPIOS.gpio_26_pulsed == 1) {
		PulsedGpioState.gpio_26 = !PulsedGpioState.gpio_26;
		gpio_set_level(GPIO_NUM_26, PulsedGpioState.gpio_26);
	} else {
		gpio_set_level(GPIO_NUM_26, AppValues.GPIOS.gpio_26_state);
	}

	if (AppValues.GPIOS.gpio_27_pulsed == 1) {
		PulsedGpioState.gpio_27 = !PulsedGpioState.gpio_27;
		gpio_set_level(GPIO_NUM_27, PulsedGpioState.gpio_27);
	} else {
		gpio_set_level(GPIO_NUM_27, AppValues.GPIOS.gpio_27_state);
	}

	if (AppValues.GPIOS.gpio_32_pulsed == 1) {
		PulsedGpioState.gpio_32 = !PulsedGpioState.gpio_32;
		gpio_set_level(GPIO_NUM_32, PulsedGpioState.gpio_32);
	} else {
		gpio_set_level(GPIO_NUM_32, AppValues.GPIOS.gpio_32_state);
	}
	if (AppValues.GPIOS.gpio_33_pulsed == 1) {
		PulsedGpioState.gpio_33 = !PulsedGpioState.gpio_33;
		gpio_set_level(GPIO_NUM_33, PulsedGpioState.gpio_33);
	} else {
		gpio_set_level(GPIO_NUM_33, AppValues.GPIOS.gpio_33_state);
	}


//		gpio_set_level(GPIO_NUM_2, 0); // STATUS LED

	//ESP_LOGI(TAG,"All gpio off display =%d I2C = %d", RunFlags.displayInUse, RunFlags.I2C_inUse);

}
