/*
 Simple udp server
 */
#include<stdio.h> //printf
#include<string.h> //memset
#include<stdlib.h> //exit(0);
#include<arpa/inet.h>
#include "esp_err.h"
#include "esp_log.h"
#include<sys/socket.h>
#include "esp_event_loop.h"
#include "driver/uart.h"

// #include <lwip/sockets.h>
#include "esp_wifi.h"

#include "k2audio.h"


#define BUF_SIZE (512)

static const char *TAG = "UDP_RX";

void udp_rx_task(void *pvParameters) {

	uint8_t *dataPtr;

	dataPtr = (uint8_t*) &udp_rec_data;

	static int s;

	bzero(dataPtr, BUF_SIZE);

	static struct sockaddr_in si_me, si_other;
	int recv_len = -1;
	uint32_t slen = sizeof(si_other);

	struct timeval read_timeout;
	read_timeout.tv_sec = 0;
	read_timeout.tv_usec = 100;


	RunFlags.pcip_acquired = false;
	s = -1;

//	while (!RunFlags.connected) {
//		vTaskDelay(500 / portTICK_PERIOD_MS);
//	}

	while (1) {
		if (s == -1) {
			// zero out the structure
			bzero((char *) &si_me, sizeof(si_me));
			//create a UDP socket
			if ((s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) != -1) {
				int err = setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &read_timeout, sizeof read_timeout);
				if (err < 0) {
					perror("Socket opt T/O fail : ");
				}

				si_me.sin_family = AF_INET;
				si_me.sin_port = htons(AppValues.IR_PORT);
				si_me.sin_addr.s_addr = htonl(INADDR_ANY);

				//bind socket to port
				if (bind(s, (struct sockaddr*) &si_me, sizeof(si_me)) == -1) {
					close(s);
					perror("UDP bind error \r\n");
					s = -1;
				}
				//ESP_LOGI(TAG,"UDP Socket created %d \r\n", s);
				ESP_LOGI(TAG,"UDP port %d \r\n", ntohs(si_me.sin_port)); // convert back to normal endian

			}

		}
		dataPtr = (uint8_t*) &udp_rec_data;

		recv_len = -1;
		recv_len = recvfrom(s, dataPtr, BUF_SIZE, 0, (struct sockaddr *) &si_other, &slen);

		if (recv_len != -1) {
			// we have data
			//print details of the client/peer and the data received
			uint8_t command, count;
			uint8_t num, ch1, ch2;

			bool found = false;
			uint8_t first = *dataPtr;
			uint8_t second = *(dataPtr + 1);
			uint8_t third = *(dataPtr + 2);
			char REPLY[8] = "ACK\r\n";

		//	memset(dataPtr+slen,0x00,0x300 - slen);
			//clear old trailing rubbish

			ESP_LOGI(TAG, "data received : %s", dataPtr);

			if ((first == 'K') && (second == ' ')) {
				//K xxx string
				RunFlags.flashACK = true;
				memcpy(&AsciiData, dataPtr, recv_len);
				dataPtr += 4;
				pairCount = ascii2hex(AsciiData[5], AsciiData[4]);
				RunFlags.IR_TX_dataReady = true;
			} else if (first == '#') {
				// #name:K
				dataPtr++;
				count = 0;
				while ((!found) && (count < 15)) {
					if (*dataPtr++ == ':') {
						found = true;
						if (*dataPtr == 'K') {
							dataPtr += 4;
							memcpy(&AsciiData, dataPtr, recv_len - count);
							pairCount = ascii2hex(*(dataPtr + 1), *dataPtr);
							//	IR_TX_dataReady = true;
						}
					}
					count++;
				}
			} else if ((first == 'c') && (second == 'm') && (third == 'd')) {
				dataPtr += 3;

				command = *dataPtr++;
				switch (command) {

				case 'B':
					// reboot
					fflush(stdout);
					esp_restart();
					break;
				case 'C':
					command = *dataPtr++;
					//cmdCH
					if (command == 'H') {
						tcpip_adapter_set_hostname(TCPIP_ADAPTER_IF_STA, (char*) dataPtr);
					} else {
						switch (command) {
						case 'I':
							//ESP_LOGI(TAG, "cmdCI received ");
							memcpy(&AppValues.pc_addr, &si_other, sizeof(si_other));
							ESP_LOGI(TAG, "address updated \r\n");
							break;
						case 'S':
							strcpy((char*) &Subnet, (char*) dataPtr);
							break;
						case 'G':
							strcpy((char*) &Gateway, (char*) dataPtr);
							break;
						case 'N':
							// name as in SSID:
							strcpy((char*) AppValues.wifiSSID, (char*) dataPtr);
							break;
						case 'K':
							// key as in password
							strcpy((char*) AppValues.wifiPass, (char*) dataPtr);
							break;
						case 'W':
							// wifi mode
							if (*dataPtr != AppValues.wifiStartMode) {
								AppValues.wifiStartMode = *dataPtr;
								restartWiFi(AppValues.wifiStartMode);
							}

							break;
						case 'P':
							//PC address
							inet_pton(AF_INET, (char*) dataPtr, &AppValues.pc_addr.sin_addr.s_addr);
							break;
						case 'T':
							// Target IP address
							inet_pton(AF_INET, (char*) dataPtr, &AppValues.target_addr.sin_addr.s_addr);
							break;
						case 'R':
							//restart WiFi!
							restartWiFi(AppValues.wifiStartMode);
							break;
						default:
							break;
							}
						}

					break;
				case 'D':
					//DHCP on / off
					if (*dataPtr == '0') {
						tcpip_adapter_dhcpc_stop(TCPIP_ADAPTER_IF_STA);
						AppValues.STORED_FLAGS.useDHCP = false;
					} else if (*dataPtr == '1') {
						tcpip_adapter_dhcpc_start(TCPIP_ADAPTER_IF_STA);
						AppValues.STORED_FLAGS.useDHCP = true;
					}
					break;
				case 'I':
					//ESP_LOGI(TAG, "cmdI received ");
					memcpy(&AppValues.pc_addr, &si_other, sizeof(si_other));
					ESP_LOGI(TAG, "address updated \r\n");
					break;
				case 'P':
					//set data port
					AppValues.IR_PORT = atoi((char*)dataPtr);
					break;
				case 'U':
					// update non volatile memory
					if (!saveAppValues()) {
						strcpy(REPLY, "Error\r\n");
					}
					break;
				default:
					strcpy(REPLY, "Error\r\n");
					break;
				}

			} else if ((first == 'A') && (second == 'C') && (third == 'K')) {
				//  ACK_LED = on;
				RunFlags.flashACK = true;
			} else {
				strcpy(REPLY, "Error\r\n");
			}

			ESP_LOGI(TAG, "Received packet from %s:%d\r\n", inet_ntoa(si_other.sin_addr), ntohs(si_me.sin_port));
			//ESP_LOGI(TAG,"Data: %s \n", dataPtr);

			memcpy(&AppValues.pc_addr, &si_other, sizeof(si_other));
			RunFlags.pcip_acquired = true;
			int sendLen = strlen(REPLY);
			if (sendto(s, REPLY, sendLen, 0, (struct sockaddr*) &si_other, slen) == -1) {
				perror("UDP rx reply sendto()");
			}
		}

		vTaskDelay(100 / portTICK_PERIOD_MS);

	}

	close(s);
	vTaskDelete( NULL);
}


