/*
 Some random cgi routines. Used in the LED example and the page that returns the entire
 flash as a binary. Also handles the hit counter on the main page.
 */

/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * Jeroen Domburg <jeroen@spritesmods.com> wrote this file. As long as you retain 
 * this notice you can do whatever you want with this stuff. If we meet some day, 
 * and you think this stuff is worth it, you can buy me a beer in return. 
 * ----------------------------------------------------------------------------
 */

#include <libesphttpd/esp.h>
#include "cgi.h"
#include "esp_log.h"
#include "os.h"
#include "k2audio.h"

#define TAG "cgi :"




///*********************POST ROUIINES*******************************
CgiStatus ICACHE_FLASH_ATTR cgiReset(HttpdConnData *connData) {
	int len;
	char buff[32];
	
	
	
	
	if (connData->isConnectionClosed) {
		//Connection aborted. Clean up.
		return HTTPD_CGI_DONE;
	}

	len = httpdFindArg(connData->getArgs, "reset_all", buff, 32);
	if (len == -1) {
		resetSystem(0, 0); // save first and reboot
		httpdRedirect(connData, "/reset.htm");
	} else {
		resetSystem(1, 1); // use defaults and reboot
		httpdRedirect(connData, "/reboot.htm");
	}


	return HTTPD_CGI_DONE;
	

}



CgiStatus ICACHE_FLASH_ATTR cgiConfig(HttpdConnData *connData) {
	int len;
	char buff[256];



	if (connData->isConnectionClosed) {
		//Connection aborted. Clean up.
		return HTTPD_CGI_DONE;

	}

	AppValues.STORED_FLAGS.useDHCP =  false;
	len = httpdFindArg(connData->post.buff, "dhcp", buff, sizeof(buff));
	if (len != -1) {
		AppValues.STORED_FLAGS.useDHCP = true;
	}


	len = httpdFindArg(connData->post.buff, "host", buff, sizeof(buff));
	if (len != -1) {
		strcpy(AppValues.HOSTNAME, buff);

	}
	len = httpdFindArg(connData->post.buff, "mip", buff, sizeof(buff));
	if (len != -1) {

		strcpy(AppValues.MyIp, buff);

	}
	len = httpdFindArg(connData->post.buff, "gw", buff, sizeof(buff));
	if (len != -1) {
		strcpy(AppValues.Gateway, buff);
	//	ESP_LOGI(TAG,"Gateway received = %s", AppValues.Gateway);

	}
	len = httpdFindArg(connData->post.buff, "sub", buff, sizeof(buff));
	if (len != -1) {
		strcpy(AppValues.Subnet, buff);

	}

	len = httpdFindArg(connData->post.buff, "DNS1", buff, sizeof(buff));
	if (len != -1) {
		strcpy(AppValues.DNS1, buff);

	}
	len = httpdFindArg(connData->post.buff, "DNS2", buff, sizeof(buff));
	if (len != -1) {

		strcpy(AppValues.DNS2, buff);
	}

	len = httpdFindArg(connData->post.buff, "port", buff, sizeof(buff));
	if (len != -1) {
		AppValues.IR_PORT = (uint16_t) atoi(buff);

	}


	len = httpdFindArg(connData->post.buff, "r2ip", buff, sizeof(buff));
	if (len != -1) {
		inet_pton(AF_INET, buff, &AppValues.pc_addr.sin_addr.s_addr);

	}


	len = httpdFindArg(connData->post.buff, "rip", buff, sizeof(buff));
	if (len != -1) {
		inet_pton(AF_INET, buff, &AppValues.target_addr.sin_addr.s_addr);

	}




	httpdRedirect(connData, "config.htm");

	RunFlags.appValuesChanged = true;
	RunFlags.resetRequired = true;
	return HTTPD_CGI_DONE;
}

//***********************GET  ROUTINES******************************

CgiStatus ICACHE_FLASH_ATTR cgiSetOptions(HttpdConnData *connData) {

	char buff[256];
	int len = 0;

	bool alsoReboot = false;

	if (connData->isConnectionClosed) {
		//Connection aborted. Clean up.
		return HTTPD_CGI_DONE;
	}

	len = httpdFindArg(connData->getArgs, "grp2", buff, sizeof(buff));
	// display
	AppValues.STORED_FLAGS.gpio_use_for_display = false;

	if (len != -1) {
		uint8_t mode;
		mode = buff[0] - '0';
		ESP_LOGI(TAG, "buff = %s mode = %d", buff, mode);

		if (mode == 1) {
			AppValues.STORED_FLAGS.gpio_use_for_display = true;
		}
	}

	len = httpdFindArg(connData->getArgs, "grp1", buff, sizeof(buff));
	// display
	if (len != -1) {
		uint8_t mode;
		mode = buff[0] - '0';
		ESP_LOGI(TAG, "buff = %s mode = %d", buff, mode);

		if (mode == 1) {
			AppValues.STORED_FLAGS.gpio_use_for_i2c = true;
		} else {
			AppValues.STORED_FLAGS.gpio_use_for_i2c = false;
		}
	}

	len = httpdFindArg(connData->getArgs, "reboot", buff, sizeof(buff));
	// LEDsr
	ESP_LOGI(TAG, "gpio reboot = %d", len);
	if (len != -1) {
		alsoReboot = true;
	}

	//*****************************Display*******************************

	len = httpdFindArg(connData->getArgs, "gpio_00S", buff, sizeof(buff));

	if (len != -1) {
		AppValues.GPIOS.gpio_00_state = true;
	} else {
		AppValues.GPIOS.gpio_00_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_00P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_00_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_00_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_05S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_05_state = true;
	} else {
		AppValues.GPIOS.gpio_05_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_05P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_05_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_05_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_13S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_13_state = true;
	} else {
		AppValues.GPIOS.gpio_13_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_13P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_13_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_13_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_14S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_14_state = true;
	} else {
		AppValues.GPIOS.gpio_14_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_14P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_14_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_14_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_15S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_15_state = true;
	} else {
		AppValues.GPIOS.gpio_15_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_15P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_15_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_15_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_19S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_19_state = true;
	} else {
		AppValues.GPIOS.gpio_19_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_19P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_19_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_19_pulsed = false;
	}

	//*************************I2C*********************************************

	len = httpdFindArg(connData->getArgs, "gpio_21S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_21_state = true;
	} else {
		AppValues.GPIOS.gpio_21_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_21P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_21_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_21_pulsed = false;
	}


	len = httpdFindArg(connData->getArgs, "gpio_22S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_22_state = true;
	} else {
		AppValues.GPIOS.gpio_22_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_22P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_22_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_22_pulsed = false;
	}

	//****************************GPIO**********************************

	len = httpdFindArg(connData->getArgs, "gpio_04S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_04_state = true;
	} else {
		AppValues.GPIOS.gpio_04_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_04P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_04_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_04_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_12S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_12_state = true;
	} else {
		AppValues.GPIOS.gpio_12_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_12P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_12_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_12_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_16S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_16_state = true;
	} else {
		AppValues.GPIOS.gpio_16_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_16P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_16_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_16_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_17S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_17_state = true;
	} else {
		AppValues.GPIOS.gpio_17_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_17P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_17_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_17_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_18S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_18_state = true;
	} else {
		AppValues.GPIOS.gpio_18_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_18P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_18_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_18_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_23S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_23_state = true;
	} else {
		AppValues.GPIOS.gpio_23_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_23P", buff, sizeof(buff));
//	ESP_LOGI(TAG, "gpio_23P  = %d", len);
	if (len != -1) {
		AppValues.GPIOS.gpio_23_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_23_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_25S", buff, sizeof(buff));
//	ESP_LOGI(TAG, "gpio_25S  = %d", len);
	if (len != -1) {
		AppValues.GPIOS.gpio_25_state = true;
	} else {
		AppValues.GPIOS.gpio_25_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_25P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_25_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_25_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_26S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_26_state = true;
	} else {
		AppValues.GPIOS.gpio_26_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_26P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_26_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_26_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_27S", buff, sizeof(buff));
		if (len != -1) {
			AppValues.GPIOS.gpio_27_state = true;
		} else {
			AppValues.GPIOS.gpio_27_state = false;
		}

	len = httpdFindArg(connData->getArgs, "gpio_27P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_27_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_27_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_32S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_32_state = true;
	} else {
		AppValues.GPIOS.gpio_32_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_32P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_32_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_32_pulsed = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_33S", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_33_state = true;
	} else {
		AppValues.GPIOS.gpio_33_state = false;
	}

	len = httpdFindArg(connData->getArgs, "gpio_33P", buff, sizeof(buff));
	if (len != -1) {
		AppValues.GPIOS.gpio_33_pulsed = true;
	} else {
		AppValues.GPIOS.gpio_33_pulsed = false;
	}

	// reset and save

	if (alsoReboot ==true){
		RunFlags.resetRequired = true;
	}
	RunFlags.appValuesChanged = true;

	httpdRedirect(connData, "/switches.htm");

	return HTTPD_CGI_DONE;

}

//************ % SUBSTITUTION ROUTINES *********************************
CgiStatus ICACHE_FLASH_ATTR tplParseSwitches(HttpdConnData *connData, char *token, void **arg) {

	if (token == NULL)
		return HTTPD_CGI_DONE;

	if (os_strcmp(token, "hostname") == 0)
		httpdSend(connData, AppValues.HOSTNAME, -1);

	updateMenu(connData, token);

	if (os_strcmp(token, "spi-display") == 0) {
		if (AppValues.STORED_FLAGS.gpio_use_for_display)
			httpdSend(connData, "checked", -1);
	}
	if (os_strcmp(token, "spi-io") == 0) {
		if (AppValues.STORED_FLAGS.gpio_use_for_display == false)
			httpdSend(connData, "checked", -1);
	}

	if (os_strcmp(token, "i2c-display") == 0) {
		if (AppValues.STORED_FLAGS.gpio_use_for_i2c)
			httpdSend(connData, "checked", -1);
	}
	if (os_strcmp(token, "i2c-io") == 0) {
		if (AppValues.STORED_FLAGS.gpio_use_for_i2c == false)
			httpdSend(connData, "checked", -1);
	}

	if (os_strcmp(token, "enable_LED") == 0) {
		if (AppValues.STORED_FLAGS.use_led) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_00S") == 0) {
		if (AppValues.GPIOS.gpio_00_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_00P") == 0) {
		if (AppValues.GPIOS.gpio_00_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_05S") == 0) {
		if (AppValues.GPIOS.gpio_05_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_05P") == 0) {
		if (AppValues.GPIOS.gpio_05_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_13S") == 0) {
		if (AppValues.GPIOS.gpio_13_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_13P") == 0) {
		if (AppValues.GPIOS.gpio_13_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_14S") == 0) {
		if (AppValues.GPIOS.gpio_14_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_14P") == 0) {
		if (AppValues.GPIOS.gpio_14_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_15S") == 0) {
		if (AppValues.GPIOS.gpio_15_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_15P") == 0) {
		if (AppValues.GPIOS.gpio_15_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_19S") == 0) {
		if (AppValues.GPIOS.gpio_19_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_19P") == 0) {
		if (AppValues.GPIOS.gpio_19_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_21S") == 0) {
		if (AppValues.GPIOS.gpio_21_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_21P") == 0) {
		if (AppValues.GPIOS.gpio_21_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_22S") == 0) {
		if (AppValues.GPIOS.gpio_22_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_22P") == 0) {
		if (AppValues.GPIOS.gpio_22_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_04S") == 0) {
		if (AppValues.GPIOS.gpio_04_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_04P") == 0) {
		if (AppValues.GPIOS.gpio_04_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_12S") == 0) {
		if (AppValues.GPIOS.gpio_12_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_12P") == 0) {
		if (AppValues.GPIOS.gpio_12_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_16S") == 0) {
		if (AppValues.GPIOS.gpio_16_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_16P") == 0) {
		if (AppValues.GPIOS.gpio_16_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_17S") == 0) {
		if (AppValues.GPIOS.gpio_17_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_17P") == 0) {
		if (AppValues.GPIOS.gpio_17_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_18S") == 0) {
		if (AppValues.GPIOS.gpio_18_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_18P") == 0) {
		if (AppValues.GPIOS.gpio_18_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_23S") == 0) {
		if (AppValues.GPIOS.gpio_23_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_23P") == 0) {
		if (AppValues.GPIOS.gpio_23_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_25S") == 0) {
	//	ESP_LOGI(TAG,"gpio_num_25S found");
		if (AppValues.GPIOS.gpio_25_state == true) {
		//	ESP_LOGI(TAG,"state 25 = true");
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_25P") == 0) {
		if (AppValues.GPIOS.gpio_25_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_26S") == 0) {
		if (AppValues.GPIOS.gpio_26_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_26P") == 0) {
		if (AppValues.GPIOS.gpio_26_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_27S") == 0) {
		if (AppValues.GPIOS.gpio_27_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_27P") == 0) {
		if (AppValues.GPIOS.gpio_27_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_32S") == 0) {
		if (AppValues.GPIOS.gpio_32_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

//	ESP_LOGI(TAG,"gpio_num_32P = %d",AppValues.GPIOS.gpio_32_pulsed  );

	if (os_strcmp(token, "gpio_num_32P") == 0) {
		if (AppValues.GPIOS.gpio_32_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_33S") == 0) {
		if (AppValues.GPIOS.gpio_33_state == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	if (os_strcmp(token, "gpio_num_33P") == 0) {
		if (AppValues.GPIOS.gpio_33_pulsed == true) {
			httpdSend(connData, "checked", -1);
		}
	}

	return HTTPD_CGI_DONE;

}

void updateMenu(HttpdConnData *connData, char *token) {

	if (os_strcmp(token,"title") == 0) {
		httpdSend(connData, "ESP32 breakout board", -1);
	}

	if (os_strcmp(token,"menuFlash") == 0) {
		httpdSend(connData, "<li><a href=\"../flash/index.html\">Flash firmware update</a></li>", -1);
	}

	if (os_strcmp(token, "MODULE-TYPE") == 0) {
		httpdSend(connData, "Developement board", -1);
	}

	if (os_strcmp(token, "WiFiMode") == 0) {
		if (AppValues.wifiStartMode == WIFI_MODE_AP) {
			httpdSend(connData, " an access point", -1);
		}
		if (AppValues.wifiStartMode == WIFI_MODE_APSTA) {
			httpdSend(connData, " an access point and station", -1);
		}
		if (AppValues.wifiStartMode == WIFI_MODE_STA) {
			httpdSend(connData, " a station", -1);
		}

	}

	if (os_strcmp(token, "menuOptions") == 0) {
			httpdSend(connData, "<li><a href=\"/switches.htm\">GPIO Options</a></li>", -1);
		}


	if (os_strcmp(token, "currAp") == 0) {
		httpdSend(connData, AppValues.wifiSSID, -1);

	}
	
	
	if (os_strcmp(token, "menuWifi") == 0) {
			if (AppValues.wifiCurrentMode == WIFI_MODE_STA) {
				httpdSend(connData, "<li><a href=\"../wifi/wifi.html\">Network</a></li>", -1);
			}else {
				httpdSend(connData, "<li><a href=\"../wifi/wifi.tpl\">Network</a></li>", -1);
			}
		}



	if (os_strcmp(token, "version") == 0)
		httpdSend(connData, version, -1);

}

CgiStatus ICACHE_FLASH_ATTR tplParseConfig(HttpdConnData *connData, char *token, void **arg) {
	char buf[32];

	if (token == NULL)
		return HTTPD_CGI_DONE;

	updateMenu(connData, token);

	if (os_strcmp(token, "config_mac") == 0)
		httpdSend(connData, AppValues.MACAsString, -1);

	if (os_strcmp(token, "config_hostname") == 0)
		httpdSend(connData, AppValues.HOSTNAME, -1);

	if (os_strcmp(token, "enable_dhcp") == 0){
		if (AppValues.STORED_FLAGS.useDHCP) {
			httpdSend(connData, "checked", -1);
		}
	}
	if (os_strcmp(token, "config_ip") == 0)
		httpdSend(connData, AppValues.MyIp, -1);

	if (os_strcmp(token, "config_sub") == 0)
		httpdSend(connData, AppValues.Subnet, -1);

	if (os_strcmp(token, "config_gw") == 0){
		httpdSend(connData, AppValues.Gateway, -1);
//		ESP_LOGI(TAG,"Gateway sent %s", AppValues.Gateway);
	}

	if (os_strcmp(token, "config_DNS1") == 0)
		httpdSend(connData, AppValues.DNS1, -1);

	if (os_strcmp(token, "config_DNS2") == 0)
		httpdSend(connData, AppValues.DNS2, -1);

	if (os_strcmp(token, "config_port") == 0) {
		sprintf(buf, "%d", AppValues.IR_PORT);
		httpdSend(connData, buf, -1);
	}
	if (os_strcmp(token, "remote1_ip") == 0) {
		strcpy(buf, inet_ntoa(AppValues.target_addr.sin_addr.s_addr));
			httpdSend(connData, buf, -1);
		}
	if (os_strcmp(token, "remote2_ip") == 0) {
		strcpy(buf, inet_ntoa(AppValues.pc_addr.sin_addr.s_addr));
		httpdSend(connData, buf, -1);
	}


	return HTTPD_CGI_DONE;
}

CgiStatus ICACHE_FLASH_ATTR tplParseIndex(HttpdConnData *connData, char *token, void **arg) {

	if (token == NULL)
		return HTTPD_CGI_DONE;

	if (os_strcmp(token, "hostname") == 0)
		httpdSend(connData, AppValues.HOSTNAME, -1);

	updateMenu(connData, token);

	return HTTPD_CGI_DONE;
}

CgiStatus ICACHE_FLASH_ATTR tplParseReset(HttpdConnData *connData, char *token, void **arg) {

	if (token == NULL)
		return HTTPD_CGI_DONE;

	if (os_strcmp(token, "hostname") == 0)
		httpdSend(connData, AppValues.HOSTNAME, -1);

	updateMenu(connData, token);

	return HTTPD_CGI_DONE;
}

