/**
 * @file lv_sysmon.c
 *
 */

/*********************
 *      INCLUDES
 *********************/
#include "display.h"
//#if LV_USE_SYSMON
#include "k2audio.h"

#include <stdio.h>

/*********************
 *      DEFINES
 *********************/
#define CPU_LABEL_COLOR     "FF0000"
#define MEM_LABEL_COLOR     "E68a00"
#define CHART_POINT_NUM     100
#define REFR_TIME    500

/**********************
 *      TYPEDEFS
 **********************/

/**********************
 *  STATIC PROTOTYPES
 **********************/

//static void win_close_action(lv_obj_t *btn, lv_event_t event);
static void display_task(lv_task_t *param);

/**********************
 *  STATIC VARIABLES
 **********************/
static lv_obj_t *win;
static lv_obj_t *chart;
static lv_chart_series_t *column0;
static lv_obj_t *info_label;
static lv_task_t *refr_task;
static uint8_t count = 0;
static char buf_long[256];
//static uint8_t rssi;
static lv_obj_t *roll;


uint8_t isPlaying;

void createRssi(void);
void createScanPicker(void);
void createTextArea(void);

/**
 * Initialize the system monitor
 */
void display_create(void) {

	refr_task = lv_task_create(display_task, REFR_TIME, LV_TASK_PRIO_LOW, NULL);


//	 lv_theme_t * th = lv_theme_get_current();

	/*Initialize the alien theme
	 * 210: a green HUE value
	 * NULL: use the default font (LV_FONT_DEFAULT)*/
	lv_theme_t *th = lv_theme_night_init(210, NULL);

	/*Set the current system theme*/
	lv_theme_set_current(th);
	win = lv_win_create(lv_disp_get_scr_act(NULL), NULL);
	lv_win_set_title(win, "K2Audio Ltd Breakout pcb demo");


	createRssi();
	createScanPicker();
	createTextArea();

	/*Refresh the chart and label manually at first*/
	display_task(NULL);
}

void createRssi() {


//	style->body.main_color = LV_COLOR_WHITE;

	chart = lv_chart_create(win, NULL);
	lv_obj_set_size(chart, 40, 40);
	lv_chart_set_div_line_count(chart, 0, 0);
	lv_obj_set_pos(chart, 270, 130);
	lv_chart_set_point_count(chart, 8);
	lv_chart_set_range(chart, 0, 20);
	lv_chart_set_type(chart, LV_CHART_TYPE_COLUMN);
	lv_chart_set_series_width(chart, 4);
	lv_chart_set_series_opa(chart, 255);

	lv_chart_set_style(chart, LV_CHART_STYLE_MAIN, &lv_style_transp);
	lv_obj_refresh_style(chart);

	column0 = lv_chart_add_series(chart, LV_COLOR_YELLOW);
	lv_chart_set_next(chart, column0, 4);
	lv_chart_set_next(chart, column0, 6);
	lv_chart_set_next(chart, column0, 8);
	lv_chart_set_next(chart, column0, 10);
	lv_chart_set_next(chart, column0, 12);
	lv_chart_set_next(chart, column0, 14);
	lv_chart_set_next(chart, column0, 16);
	lv_chart_set_next(chart, column0, 18);

}

void createScanPicker() {


	roll = lv_roller_create(win, NULL);

	lv_roller_set_style(roll, LV_ROLLER_STYLE_BG, &lv_style_transp);

	 lv_obj_set_pos(roll, 180, 0);
	 lv_obj_set_size(roll, 220,150);
	 lv_roller_set_visible_row_count(roll, 8);

	 sprintf(buf_long, "Spinner text\nExamples\nLine 3 \nEtc Etc!!");

	 lv_roller_set_align(roll, LV_LABEL_ALIGN_LEFT); // alignment of the options text.
	 lv_roller_set_options(roll, buf_long, LV_ROLLER_MODE_NORMAL);
	 lv_roller_set_selected(roll, 0x03,LV_ANIM_ON);


}
void createTextArea() {

	lv_coord_t hres = lv_disp_get_hor_res(NULL);
	lv_coord_t vres = lv_disp_get_ver_res(NULL);

	/*Create a label for the details of channels and playing*/
	info_label = lv_label_create(win, NULL);
	lv_label_set_recolor(info_label, true);


//	printf("hres= %d  vres = %d  \r\n", hres, vres);


	lv_label_set_long_mode(info_label, LV_LABEL_LONG_BREAK);
	lv_obj_set_size(info_label, hres / 2, vres); // set size AFTER long mode
	//	lv_label_set_align(info_label, LV_LABEL_ALIGN_LEFT);

}
void setDisplayTxt() {

	uint8_t stations;

	/*Refresh the windows*/

	sprintf(buf_long, "%s"LV_TXT_COLOR_CMD"%s Title Text"LV_TXT_COLOR_CMD"\n\n", "",
	MEM_LABEL_COLOR);



	stations = 10;
	count++;
	if(RunFlags.displayUpdated){
		RunFlags.displayUpdated = false;
		count = 120;
	}



	if (count == 80){
		lv_obj_set_hidden(roll, true);
		lv_obj_set_size(info_label, 320, 240);
	}

	if (count == 200){
		lv_obj_set_size(info_label, 160, 240);
		lv_obj_set_hidden(roll, false);
	}


	if (count < 40){
		sprintf(buf_long, "%sExample info stations:= %d \n"
								"IP address %s\n"
								"More: \n", buf_long, stations, AppValues.MyIp);

	} else if (count > 40 && count < 120){
		sprintf(buf_long, "%s Other useful stuff \n"
								"Variable count =  %d\n"
								"More:...", buf_long,  count);

	}else if (count > 120 && count < 200){
		sprintf(buf_long, "%sDebug \n  %s", buf_long, debugInfo);
	}



	lv_label_set_text(info_label, buf_long);

}

void updateRssi(uint8_t rssi) {

	column0->points[0] = 4;
	column0->points[1] = 6;
	column0->points[2] = 8;
	column0->points[3] = 10;
	column0->points[4] = 12;
	column0->points[5] = 14;
	column0->points[6] = 16;
	column0->points[7] = 18;

	if (rssi > 5) {
		rssi = 5;
	}

	if (rssi < 8) {
		column0->points[7] = 0;
	}

	if (rssi < 7) {
		column0->points[6] = 0;
	}

	if (rssi < 6) {
		column0->points[5] = 0;
	}

	if (rssi < 5) {
		column0->points[4] = 0;
	}
	if (rssi < 4) {
		column0->points[3] = 0;
	}

	if (rssi < 3) {
		column0->points[2] = 0;
	}

	if (rssi < 2) {
		column0->points[1] = 0;
	}

	if (rssi < 1) {
		column0->points[0] = 0;
	}

	lv_chart_refresh(chart);

}

void create_new_button(void) {

	lv_style_t style_pr;
	lv_style_copy(&style_pr, &lv_style_plain);
	style_pr.image.color = LV_COLOR_BLACK;
	style_pr.image.intense = LV_OPA_50;
	style_pr.text.color = lv_color_hex3(0xaaa);

	LV_IMG_DECLARE(black_DAB);

	lv_obj_t *imgbtn1 = lv_imgbtn_create(lv_scr_act(), NULL);
	lv_imgbtn_set_src(imgbtn1, LV_BTN_STATE_REL, &black_DAB);
	lv_imgbtn_set_src(imgbtn1, LV_BTN_STATE_PR, &black_DAB);
	lv_imgbtn_set_src(imgbtn1, LV_BTN_STATE_TGL_REL, &black_DAB);
	lv_imgbtn_set_src(imgbtn1, LV_BTN_STATE_TGL_PR, &black_DAB);
	lv_imgbtn_set_style(imgbtn1, LV_BTN_STATE_PR, &style_pr);
	lv_imgbtn_set_style(imgbtn1, LV_BTN_STATE_TGL_PR, &style_pr);
	lv_imgbtn_set_toggle(imgbtn1, true);
	lv_obj_align(imgbtn1, NULL, LV_ALIGN_CENTER, 0, -40);
	lv_obj_set_size(imgbtn1, 40, 40);

}

// static functions

/**
 * Called periodically to monitor the CPU and memory usage.
 * @param param unused
 */
static void display_task(lv_task_t *param) {

	(void) param; /*Unused*/
	count++;
	LV_LOG_TRACE("sys_mon task started");

	updateRssi(count % 8);
	isPlaying = 1;

	if (isPlaying) {
		setDisplayTxt();
	} else {
		lv_label_set_text(info_label, "Scan \n results\n \n Please use buttons to select station\n ");
		lv_roller_set_visible_row_count(roll, 8);
		lv_roller_set_align(roll, LV_LABEL_ALIGN_RIGHT);
		lv_label_set_align(info_label, LV_LABEL_ALIGN_LEFT);

	}

	LV_LOG_TRACE("sys_mon task finished");
}


/**
 * Called when the window's close button is clicked
 * @param btn pointer to the close button
 * @param event the current event
 */



