/*
 * k2audio.h
 *
 *  Created on: 28 Jun 2018
 *      Author: mark
 */

#include <libesphttpd/esp.h>
#include "cgi.h"
#include<stdio.h> //printf
#include<string.h> //memset
#include<stdlib.h> //exit(0);
#include<arpa/inet.h>
#include<sys/socket.h>
#include "esp_event_loop.h"
#include "driver/uart.h"
#include "esp_wifi.h"
#include "mdns.h"
#include "esp_int_wdt.h"
#include "driver/rmt.h"

#ifndef MAIN_UDP_REC_H_
#define MAIN_UDP_REC_H_

#define BYTE uint8_t

#define WORD uint16_t

#define MAX_FILES 61

#define DATA_SIZE 768 - 24

#define proto1
// #define production

// hardware pins used
#define STATUS_GPIO GPIO_NUM_2
#define USE_RESET  GPIO_NUM_25
#define IR_INPUT GPIO_NUM_34

// boolean

// struct


extern struct sockaddr_in si_disc;
extern struct sockaddr_in si_rs232;




typedef struct {

	char name[18];
	char ipAddr[18];
}DISCOVERY_ITEM;


DISCOVERY_ITEM discoveryList[20];
extern uint8_t discoveryTotal;



void updateMenu(HttpdConnData *connData, char *token);

// uint8_t

extern uint8_t  LEDStatus;
extern uint8_t udp_send_data[];
extern uint8_t AsciiData[];
extern uint8_t udp_send_data[];
extern uint8_t disc_data[];
extern uint8_t udp_rec_data[];
extern uint8_t rs232_data[];
extern uint32_t codeStoreData[];

extern rmt_item32_t rmtItem[];
extern rmt_item32_t rec_IR_data[];

extern uint16_t dataLen;
extern uint16_t udp_data_length;
extern uint16_t storedDataLength;

extern QueueHandle_t uart_queue;
extern char version[];
// integer

extern int pairCount;
extern int frequency;
extern int storedPairCount;
extern int locationNumber;
extern int broadcast;
extern int RS232_DataSize;

// char

extern char Gateway[];
extern char Subnet[];
extern char IPaddress[];
extern char codeName[];
extern char debugInfo[];

// bool
extern bool mDnsisUp;

// function prototypes

void stopWiFi(void);
void restartWiFi(uint8_t mode);
void setDataReady(void);
void setData2Ready(void);
void setStatic(void);
void resetSystem(bool saveFirst, bool useDefaults);
void xorit(char *str);

uint8_t ascii2hexSingle(uint8_t ascChr);
uint8_t ascii2hex(uint8_t loaByte, uint8_t highByte);

void asciitoRmt(rmt_item32_t *itemPtr);
void rmtToAscii(rmt_item32_t *itemPtr, int pos);

void udp_rx_task(void *pvParameters);
void udp_tx_task(void *pvParameters);
void blinky(void *pvParameter);
void mdns_task(void *pvParameter);
void RS232_rx_task(void *pvParameters);

void IR_tx_task();
void IR_rx_task();
void disc_rx_task(void *pvParameters);
void LVGL_task(void *pvParameters);
void RS232_tx_task(void *pvParameters);
void initPorts(void);
void http_get_task(void *pvParameters);

void initialise_mdns(void);
void mdns_print_results(mdns_result_t *results);
void query_mdns_service(const char *service_name, const char *proto);
void query_mdns_host(const char *host_name);
void mdns_check_button(void);

void start_IR(void);
void start_disc(void);
void start_rs232(void);
void buildDiscovery(uint8_t type);

void init_spiffs(void);
void writeSpiffsFile(char *fileName);
void renameSpiffsFile(char *fileName);
void readSpiffsFile(char *fileName);

int createCodeStores(uint8_t num);
bool storeCode(uint8_t num, uint32_t *data);
bool retrieveCode(uint8_t num, uint32_t *data);
bool loadAppValues(void);
bool saveAppValues(void);
bool getAndActionLocation(uint8_t num);
void parseRequest(char *url_str);
void setStartUpMode(void);
void setDefaults(void);

void byteToAsciiHex(uint8_t num, char *store);



enum LED_STATUS {
	AP_ON = 0, AP_CON, STA_ON, STA_CON
};

typedef union _WORD_VAL {
	uint16_t Val;
	uint8_t v[2];
	struct {
		uint8_t LB;
		uint8_t HB;
	} byte;
	struct {
		unsigned char b0 :1;
		unsigned char b1 :1;
		unsigned char b2 :1;
		unsigned char b3 :1;
		unsigned char b4 :1;
		unsigned char b5 :1;
		unsigned char b6 :1;
		unsigned char b7 :1;
		unsigned char b8 :1;
		unsigned char b9 :1;
		unsigned char b10 :1;
		unsigned char b11 :1;
		unsigned char b12 :1;
		unsigned char b13 :1;
		unsigned char b14 :1;
		unsigned char b15 :1;
	} bits;
} WORD_VAL, WORD_BITS;

typedef union _DWORD_VAL {
	uint32_t Val;
	uint16_t w[2];
	uint8_t v[4];
	struct {
		uint16_t LW;
		uint16_t HW;
	} word;
	struct {
		uint8_t LB;
		uint8_t HB;
		uint8_t UB;
		uint8_t MB;
	} byte;
	struct {
		WORD_VAL low;
		WORD_VAL high;
	} wordUnion;
	struct {
		unsigned char b0 :1;
		unsigned char b1 :1;
		unsigned char b2 :1;
		unsigned char b3 :1;
		unsigned char b4 :1;
		unsigned char b5 :1;
		unsigned char b6 :1;
		unsigned char b7 :1;
		unsigned char b8 :1;
		unsigned char b9 :1;
		unsigned char b10 :1;
		unsigned char b11 :1;
		unsigned char b12 :1;
		unsigned char b13 :1;
		unsigned char b14 :1;
		unsigned char b15 :1;
		unsigned char b16 :1;
		unsigned char b17 :1;
		unsigned char b18 :1;
		unsigned char b19 :1;
		unsigned char b20 :1;
		unsigned char b21 :1;
		unsigned char b22 :1;
		unsigned char b23 :1;
		unsigned char b24 :1;
		unsigned char b25 :1;
		unsigned char b26 :1;
		unsigned char b27 :1;
		unsigned char b28 :1;
		unsigned char b29 :1;
		unsigned char b30 :1;
		unsigned char b31 :1;
	} bits;
} DWORD_VAL;



typedef struct {
	uint8_t delay;
	uint8_t location;
} oneStep;

typedef struct {
	uint8_t noOfSteps;
	uint8_t currentStep;
	oneStep step[20];
	uint8_t delayCount;
} APP_MACRO;

APP_MACRO AppMacro;

enum {
	startOn = 1, startOff, startSame, startDelayed
} RELAY_STARTMODE;

typedef struct {
	bool enabled;
	uint8_t startMode;
	bool currentState;
	uint8_t delay;

} RELAY;

typedef struct {
	unsigned char gpio_00 :1;
	unsigned char gpio_05 :1;
	unsigned char gpio_13 :1;
	unsigned char gpio_14 :1;
	unsigned char gpio_15 :1;
	unsigned char gpio_19 :1;

	unsigned char gpio_21 :1;
	unsigned char gpio_22 :1;

	unsigned char gpio_04 :1;
	unsigned char gpio_12 :1;
	unsigned char gpio_16 :1;
	unsigned char gpio_17 :1;

	unsigned char gpio_18 :1;
	unsigned char gpio_23 :1;
	unsigned char gpio_25 :1;
	unsigned char gpio_26 :1;
	unsigned char gpio_27 :1;
	unsigned char gpio_32 :1;
	unsigned char gpio_33 :1;

} GPIO_STATES;

GPIO_STATES PulsedGpioState;

typedef struct __attribute__((__packed__)) {
	// config page info
	char HOSTNAME[24];
	char MyIp[16];
	char OldIP[16];
	char username[16];
	char password[16];
	char wifiSSID[20];
	char wifiPass[20];
	char wifiAP_Name[20];
	BYTE wifiStartMode;
	char Subnet[16];
	char Gateway[16];
	char DNS1[16];
	char DNS2[16];
	WORD IR_PORT;
	BYTE wifiMAC[6];
	char MACAsString[18];
	char deviceName[30];

	//network
	struct sockaddr_in pc_addr;
	struct sockaddr_in target_addr;
	struct sockaddr_in uart_addr;
	//RS232
	BYTE baudRate;
	WORD RS_PORT;
	// general
	BYTE currentMode;
	BYTE lastMode;
	BYTE route_value;
	//IR
	BYTE overRideConfiguration;
	BYTE start_compensation;
	BYTE low_compensation;
	BYTE high_compensation;
	BYTE data_valid;
	BYTE wifiCurrentMode;

	struct {
		unsigned char use_led :1;
		unsigned char static_ip :1;
		unsigned char useDHCP :1;
		unsigned char gpio_use_for_display :1;
		unsigned char gpio_use_for_i2c :1;
	} STORED_FLAGS;

	struct {
		unsigned char gpio_00_state :1;
		unsigned char gpio_00_pulsed :1;
		unsigned char gpio_05_state :1;
		unsigned char gpio_05_pulsed :1;
		unsigned char gpio_13_state :1;
		unsigned char gpio_13_pulsed :1;
		unsigned char gpio_14_state :1;
		unsigned char gpio_14_pulsed :1;
		unsigned char gpio_15_state :1;
		unsigned char gpio_15_pulsed :1;
		unsigned char gpio_19_state :1;
		unsigned char gpio_19_pulsed :1;
		unsigned char gpio_21_state :1;
		unsigned char gpio_21_pulsed :1;
		unsigned char gpio_22_state :1;
		unsigned char gpio_22_pulsed :1;
		unsigned char gpio_04_state :1;
		unsigned char gpio_04_pulsed :1;
		unsigned char gpio_12_state :1;
		unsigned char gpio_12_pulsed :1;
		unsigned char gpio_16_state :1;
		unsigned char gpio_16_pulsed :1;
		unsigned char gpio_17_state :1;
		unsigned char gpio_17_pulsed :1;
		unsigned char gpio_18_state :1;
		unsigned char gpio_18_pulsed :1;
		unsigned char gpio_23_state :1;
		unsigned char gpio_23_pulsed :1;
		unsigned char gpio_25_state :1;
		unsigned char gpio_25_pulsed :1;
		unsigned char gpio_26_state :1;
		unsigned char gpio_26_pulsed :1;
		unsigned char gpio_27_state :1;
		unsigned char gpio_27_pulsed :1;
		unsigned char gpio_32_state :1;
		unsigned char gpio_32_pulsed :1;
		unsigned char gpio_33_state :1;
		unsigned char gpio_33_pulsed :1;

	} GPIOS;

} APP_VALUES;

typedef struct {



	unsigned char debug :1;
	unsigned char IR_TX_dataReady :1;
	unsigned char IR_ST_dataReady :1;
	unsigned char connected :1;
	unsigned char pcip_acquired :1;
	unsigned char targetip_acquired :1;
	unsigned char disc_acquired :1;
	unsigned char udpDataReady :1;
	unsigned char udp2DataReady :1;
	unsigned char discDataReady :1;
	unsigned char matchFound :1;
	unsigned char matchIRinUse :1;
	unsigned char is_cmdS :1;
	unsigned char flashACK :1;
	unsigned char appValuesChanged :1;
	unsigned char resetRequired :1;
	unsigned char showFlashUpdate :1;
	unsigned char saveInProgress :1;
	unsigned char discoveryDataGenerated :1;
	unsigned char displayUpdated :1;

} RUN_FLAGS;

APP_VALUES AppValues;

RUN_FLAGS RunFlags;

#endif /* MAIN_UDP_REC_H_ */
