/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * Jeroen Domburg <jeroen@spritesmods.com> wrote this file. As long as you retain
 * this notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return.
 * ----------------------------------------------------------------------------
 */

/*
 This is example code for the esphttpd library. It's a small-ish demo showing off
 the server, including WiFi connection management capabilities, some IO etc.
 */

#include <libesphttpd/esp.h>
#include "libesphttpd/httpd.h"
#include "libesphttpd/httpdespfs.h"
#include "cgi.h"
#include "libesphttpd/cgiwifi.h"
#include "libesphttpd/cgiflash.h"
#include "libesphttpd/auth.h"
#include "libesphttpd/espfs.h"
#include "libesphttpd/captdns.h"
#include "libesphttpd/webpages-espfs.h"
#include "libesphttpd/cgiwebsocket.h"
#include "libesphttpd/httpd-freertos.h"
#include "libesphttpd/route.h"

#include "esp_wifi.h"
#include "mdns.h"

#include "k2audio.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/queue.h"

#include "freertos/event_groups.h"
#include "esp_log.h"
#include "esp_event_loop.h"
#include "nvs_flash.h"
#include "esp_event_loop.h"
#include "tcpip_adapter.h"
#include "eth_phy/phy_lan8720.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "esp_system.h"
#include "driver/gpio.h"
#include "lvgl/lvgl.h"
#include "esp_freertos_hooks.h"

#include "drv/disp_spi.h"
#include "drv/ili9341.h"
#include "drv/tp_spi.h"
#include "drv/xpt2046.h"

#include "esp_int_wdt.h"

#include "display.h"

#include "lwip/inet.h"
#include "lwip/ip4_addr.h"
#include "lwip/dns.h"

#define TAG "user_main"

// #define k2audio

#define LISTEN_PORT     80u
#define MAX_CONNECTIONS 20u

static char connectionMemory[sizeof(RtosConnType) * MAX_CONNECTIONS];
static HttpdFreertosInstance httpdFreertosInstance;

static void IRAM_ATTR lv_tick_task(void);


void 	setWifiInfo(void);



//Function that tells the authentication system what users/passwords live on the system.
//This is disabled in the default build; if you want to try it, enable the authBasic line in
//the builtInUrls below.
int myPassFn(HttpdConnData *connData, int no, char *user, int userLen, char *pass, int passLen) {
	if (no == 0) {
		strcpy(user, "admin");
		strcpy(pass, "s3cr3t");
		return 1;
//Add more users this way. Check against incrementing no for each user added.
//	} else if (no==1) {
//		strcpy(user, "user1");
//		strcpy(pass, "something");
//		return 1;
	}
	return 0;
}

#define OTA_FLASH_SIZE_K 1024
#define OTA_TAGNAME "generic"



bool bDNSFound = false;
char szURL[] = "test1.ddns.net";
ip_addr_t ip_Addr;

void dns_found_cb(const char *name, const ip_addr_t *ipaddr, void *callback_arg);


CgiUploadFlashDef uploadParams = { .type = CGIFLASH_TYPE_FW, .fw1Pos = 0x1000, .fw2Pos = ((OTA_FLASH_SIZE_K * 1024) / 2)
		+ 0x1000, .fwSize = ((OTA_FLASH_SIZE_K * 1024) / 2) - 0x1000, .tagName = OTA_TAGNAME };

/*
 This is the main url->function dispatching data struct.
 In short, it's a struct with various URLs plus their handlers. The handlers can
 be 'standard' CGI functions you wrote, or 'special' CGIs requiring an argument.
 They can also be auth-functions. An asterisk will match any url starting with
 everything before the asterisks; "*" matches everything. The list will be
 handled top-down, so make sure to put more specific rules above the more
 general ones. Authorization things (like authBasic) act as a 'barrier' and
 should be placed above the URLs they protect.
 */



HttpdBuiltInUrl builtInUrls[] = {
ROUTE_CGI_ARG("*", cgiRedirectApClientToHostname, "esp32.nonet"),
ROUTE_REDIRECT("/", "/index.tpl"),
ROUTE_CGI("/protect/configR.htm", cgiConfig),
ROUTE_CGI("/options.htm", cgiSetOptions),
ROUTE_CGI("/do-reset.htm", cgiReset),
ROUTE_TPL("/index.tpl", tplParseIndex),
ROUTE_TPL("/reset.htm", tplParseReset),
ROUTE_TPL("/switches.htm", tplParseSwitches),
ROUTE_TPL("/protect/config.htm", tplParseConfig),
ROUTE_REDIRECT("/flash", "/flash/index.html"),
ROUTE_REDIRECT("/flash/", "/flash/index.html"),
ROUTE_CGI_ARG("/flash/next", cgiGetFirmwareNext, &uploadParams),
ROUTE_CGI_ARG("/flash/upload", cgiUploadFirmware, &uploadParams),
ROUTE_CGI("/flash/reboot", cgiRebootFirmware),

//Routines to make the /wifi URL and everything beneath it work.
//Enable the line below to protect the WiFi configuration with an username/password combo.
//	{"/wifi/*", authBasic, myPassFn},

ROUTE_REDIRECT("/wifi", "/wifi/wifi.tpl"),
ROUTE_REDIRECT("/wifi/", "/wifi/wifi.tpl"),
ROUTE_CGI("/wifi/wifiscan.cgi", cgiWiFiScan),
ROUTE_TPL("/wifi/wifi.tpl", tplWlan),
ROUTE_TPL("/wifi/wifi.html", tplWlan),
ROUTE_TPL("/flash/index.html", tplParseReset),
ROUTE_CGI("/wifi/connect.cgi",	cgiWiFiConnect),
ROUTE_CGI("/wifi/connstatus.cgi", cgiWiFiConnStatus),
ROUTE_CGI("/wifi/setmode.cgi",	cgiWiFiSetMode),
ROUTE_REDIRECT("/test", "/test/index.html"),
ROUTE_REDIRECT("/test", "/test/index.html"),
ROUTE_FILESYSTEM(),
ROUTE_END() };

static EventGroupHandle_t wifi_ap_event_group;
static EventGroupHandle_t wifi_sta_event_group;
static EventGroupHandle_t eth_event_group;

/* The event group allows multiple bits for each event,
 but we only care about one event - are we connected
 to the AP with an IP? */
const static int CONNECTED_BIT = BIT0;




#define CONFIG_PHY_SMI_MDC_PIN 23
#define CONFIG_PHY_SMI_MDIO_PIN 18
#define CONFIG_PHY_ADDRESS 0

#define DEFAULT_ETHERNET_PHY_CONFIG phy_lan8720_default_ethernet_config

#define PIN_SMI_MDC   CONFIG_PHY_SMI_MDC_PIN
#define PIN_SMI_MDIO  CONFIG_PHY_SMI_MDIO_PIN



static esp_err_t wifi_event_handler(void *ctx, system_event_t *event) {

	switch (event->event_id) {
	case SYSTEM_EVENT_STA_START:
		LEDStatus = STA_ON;
		esp_wifi_connect();
		break;
	case SYSTEM_EVENT_STA_GOT_IP:
		xEventGroupSetBits(wifi_sta_event_group, CONNECTED_BIT);
		setWifiInfo();
		RunFlags.connected = true;
		LEDStatus = STA_CON;
		break;
	case SYSTEM_EVENT_STA_CONNECTED:
		ESP_LOGI(TAG, "connected \r\n");
		break;
	case SYSTEM_EVENT_STA_DISCONNECTED:
		/* This is a workaround as ESP32 WiFi libs don't currently
		 auto-reassociate. */
		esp_wifi_connect();
		xEventGroupClearBits(wifi_sta_event_group, CONNECTED_BIT);

		break;
	case SYSTEM_EVENT_AP_START: {
		tcpip_adapter_ip_info_t ap_ip_info;
		LEDStatus = AP_ON;
		if (tcpip_adapter_get_ip_info(TCPIP_ADAPTER_IF_AP, &ap_ip_info) == 0) {
			ESP_LOGI(TAG, "~~~~~~~~~~~");
			ESP_LOGI(TAG, "IP:" IPSTR, IP2STR(&ap_ip_info.ip));
			ESP_LOGI(TAG, "MASK:" IPSTR, IP2STR(&ap_ip_info.netmask));
			ESP_LOGI(TAG, "GW:" IPSTR, IP2STR(&ap_ip_info.gw));
			ESP_LOGI(TAG, "~~~~~~~~~~~");
		}

	}
		break;
	case SYSTEM_EVENT_AP_STACONNECTED:
		ESP_LOGI(TAG, "station:" MACSTR" join,AID=%d\n", MAC2STR(event->event_info.sta_connected.mac),
				event->event_info.sta_connected.aid);
		xEventGroupSetBits(wifi_ap_event_group, CONNECTED_BIT);
		LEDStatus = AP_CON;
		break;
	case SYSTEM_EVENT_AP_STADISCONNECTED:
		ESP_LOGI(TAG, "station:" MACSTR"leave,AID=%d\n", MAC2STR(event->event_info.sta_disconnected.mac),
				event->event_info.sta_disconnected.aid);
		xEventGroupClearBits(wifi_ap_event_group, CONNECTED_BIT);
		break;
	case SYSTEM_EVENT_SCAN_DONE:
		wifiScanDoneCb();
		break;
	default:
		break;
	}
	mdns_handle_system_event(ctx, event);
	return ESP_OK;
}

static esp_err_t eth_event_handler(void *ctx, system_event_t *event) {
	tcpip_adapter_ip_info_t ip;

	switch (event->event_id) {
	case SYSTEM_EVENT_ETH_CONNECTED:
		ESP_LOGI(TAG, "Ethernet Link Up");
		break;
	case SYSTEM_EVENT_ETH_DISCONNECTED:
		ESP_LOGI(TAG, "Ethernet Link Down");
		xEventGroupClearBits(eth_event_group, CONNECTED_BIT);
		break;
	case SYSTEM_EVENT_ETH_START:
		ESP_LOGI(TAG, "Ethernet Started");

		tcpip_adapter_set_hostname(TCPIP_ADAPTER_IF_STA, AppValues.HOSTNAME);

		break;
	case SYSTEM_EVENT_ETH_GOT_IP:
		RunFlags.connected = true;
		memset(&ip, 0, sizeof(tcpip_adapter_ip_info_t));
		ESP_ERROR_CHECK(tcpip_adapter_get_ip_info(ESP_IF_ETH, &ip))
		;
		ESP_LOGI(TAG, "Ethernet Got IP Addr");
		ESP_LOGI(TAG, "~~~~~~~~~~~");
		ESP_LOGI(TAG, "ETHIP:" IPSTR, IP2STR(&ip.ip));
		ESP_LOGI(TAG, "ETHMASK:" IPSTR, IP2STR(&ip.netmask));
		ESP_LOGI(TAG, "ETHGW:" IPSTR, IP2STR(&ip.gw));
		ESP_LOGI(TAG, "~~~~~~~~~~~");
		xEventGroupSetBits(eth_event_group, CONNECTED_BIT);
		break;
	case SYSTEM_EVENT_ETH_STOP:
		ESP_LOGI(TAG, "Ethernet Stopped");
		xEventGroupClearBits(eth_event_group, CONNECTED_BIT);
		break;
	default:
		break;
	}
	return ESP_OK;
}

static void eth_gpio_config_rmii(void) {
	// RMII data pins are fixed:
	// TXD0 = GPIO19
	// TXD1 = GPIO22
	// TX_EN = GPIO21
	// RXD0 = GPIO25
	// RXD1 = GPIO26
	// CLK == GPIO0
	phy_rmii_configure_data_interface_pins();
	// MDC is GPIO 23, MDIO is GPIO 18
	phy_rmii_smi_configure_pins(PIN_SMI_MDC, PIN_SMI_MDIO);
}

//connect to an access point
void init_wifi(uint8_t modeAP) {


	nvs_flash_init();

	wifi_sta_event_group = xEventGroupCreate();
	wifi_ap_event_group = xEventGroupCreate();
	wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT()
	;
	eth_event_group = xEventGroupCreate();

//Go to station mode
	esp_wifi_disconnect();

	ESP_ERROR_CHECK(esp_event_loop_init(wifi_event_handler, NULL));
	ESP_ERROR_CHECK(esp_wifi_init(&cfg));
	ESP_ERROR_CHECK(esp_wifi_set_storage(WIFI_STORAGE_RAM));
	wifi_config_t ap_config;
	strcpy((char*) (&ap_config.ap.ssid), AppValues.wifiAP_Name);
	ap_config.ap.ssid_len = strlen(AppValues.wifiAP_Name);
	ap_config.ap.channel = 1;
	ap_config.ap.authmode = WIFI_AUTH_WPA2_PSK;
	ap_config.ap.ssid_hidden = 0;
	ap_config.ap.max_connection = 20;
	strcpy((char*) &ap_config.ap.password, "K2AUDIO!");
	ap_config.ap.beacon_interval = 100;
	esp_wifi_set_config(WIFI_IF_AP, &ap_config);

	if (modeAP == WIFI_MODE_AP) { // AP
		ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_AP));
		ESP_ERROR_CHECK(esp_wifi_start());
	} else if (modeAP == WIFI_MODE_STA) {  // wifi station

		wifi_config_t config = { .sta = { .bssid_set = 0 } };
		if (AppValues.STORED_FLAGS.useDHCP == false) {
			//static ip
			ESP_LOGI(TAG,"Static IP selected DHCP flag = false");
			tcpip_adapter_dhcpc_stop(TCPIP_ADAPTER_IF_STA); // Don't run a DHCP client
			tcpip_adapter_ip_info_t ipInfo;
			inet_pton(AF_INET, AppValues.MyIp, &ipInfo.ip);
			inet_pton(AF_INET, AppValues.Gateway, &ipInfo.gw);
			inet_pton(AF_INET, AppValues.Subnet, &ipInfo.netmask);
			tcpip_adapter_set_ip_info(TCPIP_ADAPTER_IF_STA, &ipInfo);
			wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT()
			;
			ESP_ERROR_CHECK(esp_wifi_init(&cfg));
			ESP_ERROR_CHECK(esp_wifi_set_storage(WIFI_STORAGE_RAM));
			ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA));

		} else {
			//dhcp

			esp_wifi_set_mode(WIFI_MODE_STA);
			bzero(&config, sizeof(config));
		}
		sprintf((char*) config.sta.ssid, AppValues.wifiSSID);
		sprintf((char*) config.sta.password, AppValues.wifiPass);
		esp_wifi_set_config(WIFI_IF_STA, &config);
		ESP_LOGI(TAG, "Station mode starting looking for %s", config.sta.ssid);
		ESP_ERROR_CHECK(esp_wifi_start());
		tcpip_adapter_set_hostname(TCPIP_ADAPTER_IF_STA, AppValues.HOSTNAME);
		esp_wifi_connect();

	}
}
//Main routine. Initialize stdout, the I/O, filesystem and the webserver and we're done.

static void IRAM_ATTR lv_tick_task(void) {
	lv_tick_inc(portTICK_RATE_MS);
}

void app_main(void) {

	uint8_t delay_count = 0;



	RunFlags.connected = false;

	espFsInit((void*) (webpages_espfs_start));

	init_spiffs();

	initPorts();

	tcpip_adapter_init();

	httpdFreertosInit(&httpdFreertosInstance, builtInUrls,
	LISTEN_PORT, connectionMemory,
	MAX_CONNECTIONS, HTTPD_FLAG_NONE);

	setStartUpMode();
	saveAppValues();

	ESP_LOGI(TAG, "lvgl init \r\n");
	lv_init();
	disp_spi_init();
	ili9341_init();
    static lv_color_t buf1[DISP_BUF_SIZE];
    static lv_color_t buf2[DISP_BUF_SIZE];
    static lv_disp_buf_t disp_buf;
    lv_disp_buf_init(&disp_buf, buf1, buf2, DISP_BUF_SIZE);
	lv_disp_drv_t disp_drv;
	lv_disp_drv_init(&disp_drv);
	disp_drv.flush_cb = ili9341_flush;
	disp_drv.buffer = &disp_buf;
	lv_disp_drv_register(&disp_drv);
	esp_register_freertos_tick_hook(lv_tick_task);
	display_create();

// update the MAc address for the SSID decoding routine
	if (esp_efuse_mac_get_default(AppValues.wifiMAC) < 0x00) {
		perror("Get mac");

	}

	init_wifi(AppValues.wifiStartMode);
	RunFlags.debug = true;
	xTaskCreate(&blinky, "blinky", 3072, NULL, 5, NULL);
	while(!RunFlags.connected){
		lv_task_handler();
		vTaskDelay(10);
	}

	ESP_LOGI(TAG, "WiFi Starting %s %s  \r\n", AppValues.wifiSSID, AppValues.wifiPass);

	xTaskCreate(&udp_rx_task, "UDP RX task", 2048, NULL, 10, NULL);

	xTaskCreate(&udp_tx_task, "UDP TX task", 2048, NULL, 10, NULL);

	xTaskCreate(&disc_rx_task, "Discovery task", 2048, NULL, 10, NULL);

	xTaskCreate(&IR_rx_task, "IR_rx_task", 3072, NULL, 10, NULL);

	xTaskCreate(&IR_tx_task, "IR_tx_task", 3072, NULL, 10, NULL);

	ESP_LOGI(TAG, "IR receiver task started");

	xTaskCreate(&mdns_task, "mdns_task", 2048, NULL, 5, NULL);
	vTaskDelay(500 / portTICK_RATE_MS);

	ESP_LOGI(TAG, "Ready\r\n");

	/*
	// DDNS lookup test
    IP_ADDR4( &ip_Addr, 0,0,0,0 );
    printf("Get IP for URL: %s\n", szURL );
    dns_gethostbyname(szURL, &ip_Addr, dns_found_cb, NULL );
    while( !bDNSFound ) ;
    printf( "DNS found: %i.%i.%i.%i\n",
    ip4_addr1(&ip_Addr.u_addr.ip4),
    ip4_addr2(&ip_Addr.u_addr.ip4),
    ip4_addr3(&ip_Addr.u_addr.ip4),
    ip4_addr4(&ip_Addr.u_addr.ip4) );
	*/

	while(1) {
		vTaskDelay(100);
		wifi_mode_t wm;
		esp_wifi_get_mode(&wm);
		AppValues.wifiCurrentMode = wm;
		if (AppValues.wifiStartMode>WIFI_MODE_MAX){
			//Wired Ethernet
			AppValues.wifiCurrentMode = WIFI_MODE_MAX +1;
		}
		
		//start and maintain LVGL task;
		lv_task_handler();
		delay_count++; //
		if (delay_count  == 0){ //every 25.6 seconds
			setWifiInfo();

		}
	}

}


void dns_found_cb(const char *name, const ip_addr_t *ipaddr, void *callback_arg)
{
    ip_Addr = *ipaddr;
    bDNSFound = true;
}



void setWifiInfo(){
	char string[32];
	tcpip_adapter_ip_info_t MyInfo;
	uint8_t err;

	if (AppValues.wifiStartMode !=AppValues.wifiCurrentMode){
		AppValues.wifiStartMode = AppValues.wifiCurrentMode;
		saveAppValues();// so we remember and restart in current mode
	}
	///IP address- make sure we have the current assigned IP
	if (AppValues.wifiCurrentMode == WIFI_MODE_AP){
		err =tcpip_adapter_get_ip_info(TCPIP_ADAPTER_IF_AP, &MyInfo);
	}else if (AppValues.wifiCurrentMode ==WIFI_MODE_APSTA){
		err =tcpip_adapter_get_ip_info(TCPIP_ADAPTER_IF_AP, &MyInfo);
	} else if (AppValues.wifiCurrentMode == WIFI_MODE_STA){
		err =tcpip_adapter_get_ip_info(TCPIP_ADAPTER_IF_STA, &MyInfo);
	} else {
		err =tcpip_adapter_get_ip_info(TCPIP_ADAPTER_IF_ETH, &MyInfo);
	}

	if (err ==ESP_OK){
		// update IP address from current
		sprintf(string, "%s", inet_ntoa(MyInfo.ip));
		if (strcmp(AppValues.MyIp,string)!= 0){
			strcpy(AppValues.MyIp,string);
			ESP_LOGI(TAG,"IP address updated %s",AppValues.MyIp);
		}
	}

}

void resetSystem(bool saveFirst, bool useDefaults) {

	ESP_LOGI(TAG, "System reset %d  %d ", saveFirst, useDefaults);
	esp_wifi_disconnect();
	vTaskDelay(100 / portTICK_RATE_MS);
	esp_wifi_stop();

	if (useDefaults){
		saveFirst =true;
		AppValues.data_valid = 0xFF;
	}

	if (saveFirst) {
		RunFlags.saveInProgress = true;
		saveFirst = saveAppValues();
		vTaskDelay(100);
		while (RunFlags.saveInProgress == true){
		}
		esp_restart();

	} else {
		if (RunFlags.saveInProgress){
			while (RunFlags.saveInProgress == true){
				// just in case something else started a save AppValues
			}
		}

		esp_restart();
	}

}

void restartWiFi(uint8_t mode) {
	esp_wifi_stop();
	vTaskDelay(1000 / portTICK_RATE_MS);
	init_wifi(AppValues.wifiStartMode);
}
